/*                                                                            */

/*----------------------------------------------------------------------------*/
/*                                                                            */
/*----------------------------------------------------------------------------*/

	#include <proto/exec.h>
	#include <proto/dos.h>
	#include <dos/dostags.h>

/*----------------------------------------------------------------------------*/
/*                                                                            */
/*----------------------------------------------------------------------------*/

	struct bginitmsg {
		struct Message msg;
		char *text;
		char *result;
		};

/*----------------------------------------------------------------------------*/
/*                                                                            */
/*----------------------------------------------------------------------------*/

	__saveds
	void background_task (void)

	{
	struct Process *pr = (struct Process *) FindTask (NULL);
	struct bginitmsg *initmsg;
	BPTR win;

	WaitPort (&pr->pr_MsgPort);
	initmsg = (struct bginitmsg *) GetMsg (&pr->pr_MsgPort);

	initmsg->result = "failed to open window";

	if (win = Open ("CON://400/200/Background Task/INACTIVE",MODE_NEWFILE))
		{
		int i;

		initmsg->result = "success";

		for (i = 10; i > 0; i--)
			{
			FPrintf (win,"processing %s ... %ld to go\n",initmsg->text,i);
			Delay (TICKS_PER_SECOND);

			if (SetSignal (0,0) & SIGBREAKF_CTRL_C)
				{
				initmsg->result = "stopped by break signal";
				break;
				}
			}

		Close (win);
		}

	Forbid();
	ReplyMsg (&initmsg->msg);
	}

/*----------------------------------------------------------------------------*/
/*                                                                            */
/*----------------------------------------------------------------------------*/

	struct Process *start_background_task (struct MsgPort *replyport,char *text)

	{
	struct Process *pr = NULL;
	struct bginitmsg *initmsg = AllocVec (sizeof(struct bginitmsg),MEMF_CLEAR|MEMF_PUBLIC);
	if (initmsg)
		{
		if (pr = CreateNewProcTags (
				NP_Name,"example_background_task",
				NP_Entry,background_task,
				TAG_END))
			{
			initmsg->msg.mn_ReplyPort = replyport;
			initmsg->text = text;
			PutMsg (&pr->pr_MsgPort,&initmsg->msg);
			}
		}

	return (pr);
	}

/*----------------------------------------------------------------------------*/
/*                                                                            */
/*----------------------------------------------------------------------------*/

	char *stop_background_task (struct MsgPort *replyport,struct Process *pr)

	{
	struct bginitmsg *msg;
	char *result;

	msg = (struct bginitmsg *) GetMsg (replyport);	/* get reply from backgroung task */

	if (!msg)	/* reply not yet sent */
		{
		Signal (&pr->pr_Task,SIGBREAKF_CTRL_C);	/* send break signal to background task */
		do	{
			Delay (TICKS_PER_SECOND / 10);		/* avoid busy-waiting */
			msg = (struct bginitmsg *) GetMsg (replyport);	/* check if response is here now */
			}
		while (!msg);
		}

	result = msg->result;	/* get result from message */

	FreeVec (msg);		/* free message memory */

	return (result);
	}

/*----------------------------------------------------------------------------*/
/*                                                                            */
/*----------------------------------------------------------------------------*/

	int main (void)

	{
	struct MsgPort *replyport = CreateMsgPort();	/* this port receives a message when the background task ends */
	if (replyport)
		{
		struct Process *back_pr = start_background_task (replyport,"test data");
		if (back_pr)
			{
			ULONG portsig = 1L << replyport->mp_SigBit;
			char *result;

			Printf ("Background task started; waiting for completion.\n"
					"Press Ctrl-C to cancel processing.\n");

			Wait (portsig | SIGBREAKF_CTRL_C);

			result = stop_background_task (replyport,back_pr);

			Printf ("Background task stopped.\nResult: %s\n",result);
			}

		DeleteMsgPort (replyport);
		}

	return (0);
	}

/*----------------------------------------------------------------------------*/
/*                                                                            */
/*----------------------------------------------------------------------------*/
